/***************************************************************************

	File: pic_delay.c

	Copyright (C) 2010-2012 S.Suwa <www.suwa-koubou.jp>

	for 16Mhz
		PIC24FJ64/PIC24FJ128/PIC24FJ256
	for 40Mhz|80Mhz
		PIC32MX220F/PIC32MX695F	(use TMR5)

*/

#include	"pic_delay.h"
#include	"HardwareProfile.h"

#if defined(__PIC24F__)

// PIC24F 16MHz clock cycle

/*--------------------------------------------------------------------------
	delay routines
--------------------------------------------------------------------------*/

void delay_ms(unsigned int t)
{
  unsigned int i;

  for (i=0; i<t; i++) {
	asm("repeat #15993");  // for 16MHz clock
	asm("nop");
  }  
}


void delay_10us(unsigned int t)
{
  unsigned int i;

  for (i=0; i<t; i++) {
    asm("repeat #153");  // for 16MHz clock
    asm("nop");
  }  
}


void delay_us(unsigned int t)
{
	unsigned int i;

	for(i=0; i < t; i++){
		asm("repeat #10");	// for 16Mhz clock
		asm("nop");
	}
}


void delay_5us(void)
{
	delay_1us(); delay_1us(); delay_1us(); delay_1us(); 
	asm("repeat #7");	// repeat 1cycle
	asm("nop");			// nop 1cycle * 8(#7+1)
}

void delay_1us(void)
{
	// call 2cycle
	// ret  3cycle
	// lnk  1cycle
	// ulnk	1cycle
	// nop  1cycle x 9
	// total 16cycle, instruction 16cycle/usec

	asm("repeat #7");	// repeat 1cycle
	asm("nop");			// nop 1cycle * 8(#7+1)

}


#elif defined(__PIC32MX__)
#include	<plib.h>

// PIC32MX220/PIC32MX695F

/*--------------------------------------------------------------------------
	delay routines
--------------------------------------------------------------------------*/

void delay_ms(unsigned int t)
{
	unsigned int	limit;

	T5CON = 0x8010;	// prescale 1:2
	limit = GetPeripheralClock()/2/1000;	// count of 1 msec = (40MHz|80MHz)/2/1000

	while(t--){
		TMR5 = 0;	// reset timer
		while(TMR5 < limit);	// wait 1msec
	}
	T5CON = 0;
}


void delay_10us(unsigned int t)
{
	while(t--){
		delay_5us();
		delay_5us();
	}
}


void delay_us(unsigned int t)
{
	while(t--){
		delay_1us();
	}
}


void delay_5us(void)
{
	delay_1us();
	delay_1us();
	delay_1us();
	delay_1us(); 
	delay_1us(); 
}

void delay_1us(void)
{
	T5CON = 0x8000;	// prescale 1:1
	TMR5 = 0;
	while(TMR5 < (GetPeripheralClock()/1000000UL -5) );
	T5CON = 0;
}

#endif

/*** end of pic_delay.c ****************************************************/
